pub fn from_json_str<'a, T>(s: &'a str) -> Result<T, Error>where
T: Deserialize<'a>,Available on crate feature
sonic-rs only.Expand description
Deserialize an instance of type T from a string of JSON text.
pub fn from_json_str<'a, T>(s: &'a str) -> Result<T, Error>where
T: Deserialize<'a>,sonic-rs only.Deserialize an instance of type T from a string of JSON text.